/*
	ZillionServer.h
	Tue Aug 27 17:00:11 CEST 2002
 */

#import <Foundation/Foundation.h>
#import	"ZillionClientServer.h"
#import "ZillionClient.h"

#define	ZILLION_SERVER_NAME	@"zillion"

@protocol ZillionClientProtocol;
@protocol ZillionServerProtocol;
@protocol ZillionJobProtocol;

@protocol ZillionServerProtocol <NSObject>

/*
	managing masters
 */
- (void)setMasterServer:(id <ZillionServerProtocol>)masterServer;
- (id <ZillionServerProtocol>)masterServer;
- (void)terminate;

/*
	managing clients
 */
- (void)registerClient:(id <ZillionClientProtocol>)someClient;
- (void)deregisterClient:(id <ZillionClientProtocol>)someClient;


/*
	managing jobs
 */

- (NSString *)dispatchJob:(bycopy NSData *)bundle;
- (bycopy NSString *)loadAndRegisterJob:(NSData *)bundle;

- (void)registerJob:(NSData *)jobBundle withId:(NSString *)jobId;

- (NSData *)packedJobBundleForJobId:(NSString *)jobId;
/*
	forward outcome to the client to let it store relevant stuff
	if outcome cannot be submitted as a linge chunk at the end of the job unit
	someStoreHint can be used to 
 */
- (void)submitOutcome:(NSData *)outcome ofJob:(NSString *)jobId forSeedId:(NSData *)seedId
	storeHints:(NSDictionary *)someStoreHint;

- (void)jobUnitDidEndForJobId:(NSString *)jobId seedId:(NSData *)seedId
	timePenalty:(NSTimeInterval)penalty;

/*
	bringing the jobs into action
 */

// sender is a dummy argument to make it amenable to performSelector:
- (void)scheduleJobs:sender;
- (NSData *)nextSeedIdForJobId:(NSString *)jobId;

/*
	storing results
 */
- (void)setOutcome:(bycopy NSData *)outcome forSeedId:(bycopy NSData *)seedId
	andJobId:(bycopy NSString *)jobId;
- (NSMutableDictionary *)outcomeDictionaryForJobId:(bycopy NSString *)jobId;

@end

#define	ZillionClientKey	@"client"
#define	ZillionSeedKey		@"seed"
#define	ZillionJobKey		@"job"

@protocol ZillionJobProtocol <NSObject>

+ (NSString *)jobId;
+ (NSData *)dataRepresentationOfSeed:seed;
+ seedObjectFromDataRepresentation:(NSData *)seed;
- initWithBundle:(NSBundle *)someBundle;

- (NSEnumerator *)seedEnumeratorWithSeed:(NSData *)seed;


/*
	The specification of the jobs entails the following keys (with ovious meanings)
	ZillionClientKey
	ZillionSeedKey
 */
- (void)startJobUnitWithSeed:aSeed andClient:(id <ZillionClientProtocol>)aClient;

/*
	The job class must not store any information in global variables since the class
	may be deallocated at any time. Instead the information can be stored in the server
	via -setOutcome:forSeedId:andJobId:
 */
+ (void)saveOutcome:(NSData *)outcome forSeedId:(NSData *)seedId
	storeHints:(NSDictionary *)someStoreHint usingServer:(id <ZillionServerProtocol>)aServer;
+ (void)sealJobOutcomesUsingServer:(id <ZillionServerProtocol>)aServer;

@end

@interface ZillionServer : ZillionClientServer <ZillionServerProtocol> 
{

	NSMutableDictionary	*proxyServers;
	unsigned			proxyServerIdCounter;

	NSMutableDictionary	*clients;
	NSMutableDictionary	*dispatchedSeeds;
	NSMutableDictionary	*outcomes;

	BOOL	_serverToGoDown;
	BOOL	_respawnStatus;
	BOOL	_reschedulingIsMarked;
}

- initWithRespawnStatus:(BOOL)respawnStatus;
- (BOOL)serverToGoDown;

- (id <ZillionJobProtocol>)jobInstance:(NSString *)jobId;

@end
